@@ -350,6 +350,23 @@ const test_t test[] = {
     },
     IDN2_OK
   },
+  {
+    "Empty label",
+    "xn---.de",
+    {
+      0x2e, 0x64, 0x65, 0
+    },
+    IDN2_PUNYCODE_BAD_INPUT
+  },
+  {
+    "No ASCII char but delimiter",
+    "xn---tda.de",
+    {
+      0x2e, 0x64, 0x65, 0
+    },
+    IDN2_PUNYCODE_BAD_INPUT
+  },
+
 };
 
 static int debug = 1;
@@ -362,7 +379,7 @@ fail (const char *format, ...)
   va_list arg_ptr;
 
   va_start (arg_ptr, format);
-  vfprintf (stderr, format, arg_ptr);
+  vprintf (format, arg_ptr);
   va_end (arg_ptr);
 
   error_count++;
@@ -408,7 +425,7 @@ _check_4z(const test_t *t, int rc, uint32_t *ucs4, const char *funcname)
 {
   if (rc != t->rc_expected && !(rc == IDN2_ICONV_FAIL && t->rc_expected == IDN2_ENCODING_ERROR))
     {
-      fprintf (stderr, "Test[%u] '%s' failed (got %d, expected %d):\n",
+      printf ("Test[%u] '%s' failed (got %d, expected %d):\n",
         (unsigned) (t - test), t->name, rc, t->rc_expected);
       fail ("  %s(): %s\n", funcname, idn2_strerror (rc));
     }
